"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var axios_1 = require("axios");
var logspect_1 = require("logspect");
/**
 * Indicates whether the request was a success or not (between 200-300).
 */
function isOkay(response) {
    return response.status >= 200 && response.status < 300;
}
/**
 * Determines whether an object is a DavenportError.
 */
function isDavenportError(error) {
    return error.isDavenport;
}
exports.isDavenportError = isDavenportError;
/**
 * A generic view document for listing and counting all objects in the database.
 */
exports.GENERIC_LIST_VIEW = {
    "name": "all",
    "map": function (doc) { emit(doc._id, doc); }.toString(),
    "reduce": "_count"
};
/**
 * Configures an instance of Axios with auth and validation defaults.
 */
function getAxiosInstance(options) {
    var auth;
    if (options && (options.username || options.password)) {
        auth = {
            username: options.username,
            password: options.password,
        };
    }
    var instance = axios_1.default.create({
        // Like fetch, Axios should never throw an error if it receives a response
        validateStatus: function (status) { return true; },
        auth: auth,
        proxy: options && options.proxy
    });
    return instance;
}
/**
 * Configures a Davenport client and database by validating the CouchDB version, creating indexes and design documents, and then returning a client to interact with the database.
 */
function configureDatabase(databaseUrl, configuration, options) {
    return __awaiter(this, void 0, void 0, function () {
        var _this = this;
        var ax, dbInfo, infoBody, version, putResult, preconditionFailed, data, result;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    ax = getAxiosInstance(options);
                    return [4 /*yield*/, ax.get(databaseUrl)];
                case 1:
                    dbInfo = _a.sent();
                    if (!isOkay(dbInfo)) {
                        throw new Error("Failed to connect to CouchDB instance at " + databaseUrl + ". " + dbInfo.status + " " + dbInfo.statusText);
                    }
                    infoBody = dbInfo.data;
                    version = parseInt(infoBody.version);
                    if (version < 2) {
                        logspect_1.default("Warning: Davenport expects your CouchDB instance to be running CouchDB 2.0 or higher. Version detected: " + version + ". Some database methods may not work.");
                    }
                    return [4 /*yield*/, ax.put(databaseUrl + "/" + configuration.name, undefined)];
                case 2:
                    putResult = _a.sent();
                    preconditionFailed = 412;
                    if (putResult.status !== preconditionFailed && !isOkay(putResult)) {
                        throw new DavenportError(putResult.status + " " + putResult.statusText + " " + putResult.data, putResult);
                    }
                    if (!(Array.isArray(configuration.indexes) && configuration.indexes.length > 0)) return [3 /*break*/, 4];
                    data = {
                        index: {
                            fields: configuration.indexes
                        },
                        name: configuration.name + "-indexes",
                    };
                    return [4 /*yield*/, ax.post(databaseUrl + "/" + configuration.name + "/_index", data, {
                            headers: {
                                "Content-Type": "application/json"
                            },
                        })];
                case 3:
                    result = _a.sent();
                    if (!isOkay(result)) {
                        throw new DavenportError("Error creating CouchDB indexes on database " + configuration.name + ".", result);
                    }
                    _a.label = 4;
                case 4:
                    if (!(Array.isArray(configuration.designDocs) && configuration.designDocs.length > 0)) return [3 /*break*/, 6];
                    return [4 /*yield*/, Promise.all(configuration.designDocs.map(function (designDoc) { return __awaiter(_this, void 0, void 0, function () {
                            var url, getDoc, okay, docFromDatabase, docViews, shouldUpdate, result;
                            return __generator(this, function (_a) {
                                switch (_a.label) {
                                    case 0:
                                        url = databaseUrl + "/" + configuration.name + "/_design/" + designDoc.name;
                                        return [4 /*yield*/, ax.get(url)];
                                    case 1:
                                        getDoc = _a.sent();
                                        okay = isOkay(getDoc);
                                        if (!isOkay && getDoc.status !== 404) {
                                            logspect_1.default("Davenport: Failed to retrieve design doc \"" + designDoc.name + "\". " + getDoc.status + " " + getDoc.statusText, getDoc.data);
                                            return [2 /*return*/];
                                        }
                                        if (!isOkay) {
                                            docFromDatabase = {
                                                _id: "_design/" + designDoc.name,
                                                _rev: undefined,
                                                language: "javascript",
                                                views: {}
                                            };
                                        }
                                        else {
                                            docFromDatabase = getDoc.data;
                                        }
                                        docViews = designDoc.views;
                                        shouldUpdate = false;
                                        docViews.forEach(function (view) {
                                            if (!docFromDatabase.views || !docFromDatabase.views[view.name] || docFromDatabase.views[view.name].map !== view.map || docFromDatabase.views[view.name].reduce !== view.reduce) {
                                                docFromDatabase.views = Object.assign({}, docFromDatabase.views, (_a = {},
                                                    _a[view.name] = {
                                                        map: view.map,
                                                        reduce: view.reduce,
                                                    },
                                                    _a));
                                                shouldUpdate = true;
                                            }
                                            var _a;
                                        });
                                        if (!shouldUpdate) return [3 /*break*/, 3];
                                        logspect_1.default("Davenport: Creating or updating design doc \"" + designDoc.name + "\" for database \"" + configuration.name + "\".");
                                        return [4 /*yield*/, ax.put(url, docFromDatabase, {
                                                headers: {
                                                    "Content-Type": "application/json",
                                                }
                                            })];
                                    case 2:
                                        result = _a.sent();
                                        if (!isOkay(result)) {
                                            logspect_1.default("Davenport: Could not create or update CouchDB design doc \"" + designDoc.name + "\" for database \"" + configuration.name + "\". " + result.status + " " + result.statusText, result.data);
                                        }
                                        _a.label = 3;
                                    case 3: return [2 /*return*/, Promise.resolve()];
                                }
                            });
                        }); }))];
                case 5:
                    _a.sent();
                    _a.label = 6;
                case 6: return [2 /*return*/, new Client(databaseUrl, configuration.name, options)];
            }
        });
    });
}
exports.configureDatabase = configureDatabase;
/**
 * A client for interacting with a CouchDB instance. Use this when you don't want or need to use the `configureClient` function to create a database and set up design docs or indexes.
 */
var Client = (function () {
    function Client(baseUrl, databaseName, options) {
        if (options === void 0) { options = { warnings: true }; }
        this.baseUrl = baseUrl;
        this.databaseName = databaseName;
        this.options = options;
        this.databaseUrl = baseUrl + "/" + databaseName + "/";
        this.axios = getAxiosInstance(options);
    }
    Object.defineProperty(Client.prototype, "jsonContentTypeHeaders", {
        get: function () {
            return {
                "Content-Type": "application/json"
            };
        },
        enumerable: true,
        configurable: true
    });
    Client.prototype.getOption = function (name) {
        if (!this.options) {
            return undefined;
        }
        return this.options[name];
    };
    /**
     * Checks that the Axios response is okay. If not, a DavenPort error is thrown.
     */
    Client.prototype.checkErrorAndGetBody = function (result) {
        return __awaiter(this, void 0, void 0, function () {
            var message;
            return __generator(this, function (_a) {
                if (!isOkay(result)) {
                    message = "Error with " + result.config.method + " request for CouchDB database " + this.databaseName + " at " + result.config.url + ". " + result.status + " " + result.statusText;
                    throw new DavenportError(message, result);
                }
                return [2 /*return*/, result.data];
            });
        });
    };
    ;
    /**
     * Find matching documents according to the selector.
     */
    Client.prototype.find = function (options) {
        return __awaiter(this, void 0, void 0, function () {
            var result, body;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.axios.post(this.databaseUrl + "/_find", options, {
                            headers: this.jsonContentTypeHeaders
                        })];
                    case 1:
                        result = _a.sent();
                        return [4 /*yield*/, this.checkErrorAndGetBody(result)];
                    case 2:
                        body = _a.sent();
                        if (body.warning && !!this.getOption("warnings")) {
                            logspect_1.default("Davenport warning: Davenport.find result contained warning:", body.warning);
                        }
                        return [2 /*return*/, body.docs];
                }
            });
        });
    };
    /**
     * Lists documents in the database. Warning: this result WILL list design documents, and it will force the `include_docs` option to false. If you need to include docs, use .listWithDocs.
     */
    Client.prototype.listWithoutDocs = function (options) {
        if (options === void 0) { options = {}; }
        return __awaiter(this, void 0, void 0, function () {
            var result, body;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.axios.get(this.databaseUrl + "/_all_docs", {
                            params: __assign({}, this.encodeOptions(options), { include_docs: false })
                        })];
                    case 1:
                        result = _a.sent();
                        return [4 /*yield*/, this.checkErrorAndGetBody(result)];
                    case 2:
                        body = _a.sent();
                        return [2 /*return*/, {
                                offset: body.offset,
                                total_rows: body.total_rows,
                                rows: body.rows.map(function (r) { return r.value; })
                            }];
                }
            });
        });
    };
    /**
     * Lists documents in the database. Warning: this result WILL list design documents, and it will force the `include_docs` option to true. If you don't need to include docs, use .listWithoutDocs.
     */
    Client.prototype.listWithDocs = function (options) {
        if (options === void 0) { options = {}; }
        return __awaiter(this, void 0, void 0, function () {
            var result, body;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.axios.get(this.databaseUrl + "/_all_docs", {
                            params: __assign({}, this.encodeOptions(options), { include_docs: true })
                        })];
                    case 1:
                        result = _a.sent();
                        return [4 /*yield*/, this.checkErrorAndGetBody(result)];
                    case 2:
                        body = _a.sent();
                        return [2 /*return*/, {
                                offset: body.offset,
                                total_rows: body.total_rows,
                                rows: body.rows.map(function (r) { return r.doc; })
                            }];
                }
            });
        });
    };
    /**
     * Counts all documents in the database. Warning: this result WILL include design documents.
     */
    Client.prototype.count = function () {
        return __awaiter(this, void 0, void 0, function () {
            var result, body;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.axios.get(this.databaseUrl + "/_all_docs", {
                            params: {
                                limit: 0,
                            }
                        })];
                    case 1:
                        result = _a.sent();
                        return [4 /*yield*/, this.checkErrorAndGetBody(result)];
                    case 2:
                        body = _a.sent();
                        return [2 /*return*/, body.total_rows];
                }
            });
        });
    };
    Client.prototype.countBySelector = function (selector) {
        return __awaiter(this, void 0, void 0, function () {
            var result;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.find({
                            fields: ["_id"],
                            selector: selector,
                        })];
                    case 1:
                        result = _a.sent();
                        return [2 /*return*/, result.length];
                }
            });
        });
    };
    /**
     * Gets a document with the given id and optional revision id.
     */
    Client.prototype.get = function (id, rev) {
        return __awaiter(this, void 0, void 0, function () {
            var result, body;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.axios.get(this.databaseUrl + id, {
                            params: { rev: rev }
                        })];
                    case 1:
                        result = _a.sent();
                        return [4 /*yield*/, this.checkErrorAndGetBody(result)];
                    case 2:
                        body = _a.sent();
                        return [2 /*return*/, body];
                }
            });
        });
    };
    /**
     * Inserts, updates or deletes multiple documents at the same time.
     *
     * Omitting the `_id` property from a document will cause CouchDB to generate the id itself.
     *
     * When updating a document, the `_rev` property is required.
     *
     * To delete a document, set the `_deleted` property to `true`.
     *
     * Note that CouchDB will return in the response an id and revision for every document passed as content to a bulk insert, even for those that were just deleted.
     *
     * If the `_rev` does not match the current version of the document, then that particular document will not be saved and will be reported as a conflict, but this does not prevent other documents in the batch from being saved.
     *
     * If the `newEdits` arg is `false` (to push existing revisions instead of creating new ones) the response will not include entries for any of the successful revisions (since their rev IDs are already known to the sender), only for the ones that had errors. Also, the `"conflict"` error will never appear, since in this mode conflicts are allowed.
     *
     * @param docs An array of documents that will be inserted, updated or deleted.
     * @param newEdits A boolean that determines whether to allow new edits or not.
     */
    Client.prototype.bulk = function (docs, newEdits) {
        if (newEdits === void 0) { newEdits = true; }
        return __awaiter(this, void 0, void 0, function () {
            var result, body;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.axios.post(this.databaseUrl + "_bulk_docs", { docs: docs }, {
                            headers: this.jsonContentTypeHeaders,
                            params: { new_edits: newEdits }
                        })];
                    case 1:
                        result = _a.sent();
                        return [4 /*yield*/, this.checkErrorAndGetBody(result)];
                    case 2:
                        body = _a.sent();
                        return [2 /*return*/, body];
                }
            });
        });
    };
    /**
     * Creates a document with a random id. By CouchDB convention, this will only return the id and revision id of the new document, not the document itself.
     */
    Client.prototype.post = function (data) {
        return __awaiter(this, void 0, void 0, function () {
            var result, body;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.axios.post(this.databaseUrl, data, {
                            headers: this.jsonContentTypeHeaders
                        })];
                    case 1:
                        result = _a.sent();
                        return [4 /*yield*/, this.checkErrorAndGetBody(result)];
                    case 2:
                        body = _a.sent();
                        return [2 /*return*/, {
                                id: body.id,
                                rev: body.rev,
                            }];
                }
            });
        });
    };
    /**
     * Updates or creates a document with the given id. By CouchDB convention, this will only return the id and revision id of the new document, not the document itself.
     */
    Client.prototype.put = function (id, data, rev) {
        return __awaiter(this, void 0, void 0, function () {
            var result, body;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!rev && !!this.getOption("warnings")) {
                            logspect_1.default("Davenport warning: no revision specified for Davenport.put function with id " + id + ". This may cause a document conflict error.");
                        }
                        return [4 /*yield*/, this.axios.put(this.databaseUrl + id, data, {
                                headers: this.jsonContentTypeHeaders,
                                params: { rev: rev }
                            })];
                    case 1:
                        result = _a.sent();
                        return [4 /*yield*/, this.checkErrorAndGetBody(result)];
                    case 2:
                        body = _a.sent();
                        return [2 /*return*/, {
                                id: body.id,
                                rev: body.rev,
                            }];
                }
            });
        });
    };
    /**
     * Copies the document with the given id and assigns the new id to the copy. By CouchDB convention, this will only return the id and revision id of the new document, not the document itself.
     */
    Client.prototype.copy = function (id, newId) {
        return __awaiter(this, void 0, void 0, function () {
            var result, body;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.axios.request({
                            url: this.databaseUrl + id,
                            method: "COPY",
                            headers: {
                                Destination: newId
                            },
                        })];
                    case 1:
                        result = _a.sent();
                        return [4 /*yield*/, this.checkErrorAndGetBody(result)];
                    case 2:
                        body = _a.sent();
                        return [2 /*return*/, {
                                id: body.id,
                                rev: body.rev,
                            }];
                }
            });
        });
    };
    /**
     * Deletes the document with the given id and revision id.
     */
    Client.prototype.delete = function (id, rev) {
        return __awaiter(this, void 0, void 0, function () {
            var result;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!rev && !!this.getOption("warnings")) {
                            logspect_1.default("Davenport warning: no revision specified for Davenport.delete function with id " + id + ". This may cause a document conflict error.");
                        }
                        return [4 /*yield*/, this.axios.delete(this.databaseUrl + id, {
                                params: { rev: rev }
                            })];
                    case 1:
                        result = _a.sent();
                        return [4 /*yield*/, this.checkErrorAndGetBody(result)];
                    case 2:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Checks that a document with the given id exists.
     */
    Client.prototype.exists = function (id) {
        return __awaiter(this, void 0, void 0, function () {
            var result;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.axios.head(this.databaseUrl + id)];
                    case 1:
                        result = _a.sent();
                        return [2 /*return*/, result.status === 200];
                }
            });
        });
    };
    /**
     * Checks that a document that matches the field value exists.
     */
    Client.prototype.existsByFieldValue = function (value, field) {
        return __awaiter(this, void 0, void 0, function () {
            var findResult, _a;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0: return [4 /*yield*/, this.find({
                            fields: ["_id"],
                            limit: 1,
                            selector: (_a = {},
                                _a[field] = value,
                                _a)
                        })];
                    case 1:
                        findResult = _b.sent();
                        return [2 /*return*/, findResult.length > 0];
                }
            });
        });
    };
    /**
     * Checks that a document matching the selector exists.
     */
    Client.prototype.existsBySelector = function (selector) {
        return __awaiter(this, void 0, void 0, function () {
            var findResult;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.find({
                            fields: ["_id"],
                            limit: 1,
                            selector: selector,
                        })];
                    case 1:
                        findResult = _a.sent();
                        return [2 /*return*/, findResult.length > 0];
                }
            });
        });
    };
    /**
     * Executes a view with the given designDocName and viewName. Will not reduce by default, pass in the { reduce: true } option to reduce.
     */
    Client.prototype.view = function (designDocName, viewName, options) {
        if (options === void 0) { options = { reduce: false }; }
        return __awaiter(this, void 0, void 0, function () {
            var result, body;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        // Ensure reduce is set to false unless explicitly set by the caller.
                        if (typeof (options.reduce) !== "boolean") {
                            options.reduce = false;
                        }
                        return [4 /*yield*/, this.axios.get(this.databaseUrl + "_design/" + designDocName + "/_view/" + viewName, {
                                params: this.encodeOptions(options),
                            })];
                    case 1:
                        result = _a.sent();
                        return [4 /*yield*/, this.checkErrorAndGetBody(result)];
                    case 2:
                        body = _a.sent();
                        return [2 /*return*/, body];
                }
            });
        });
    };
    /**
     * Executes a view with the given designDocName and viewName. This method will never reduce the result.
     */
    Client.prototype.viewWithDocs = function (designDocName, viewName, options) {
        if (options === void 0) { options = { reduce: false }; }
        return __awaiter(this, void 0, void 0, function () {
            var result, body;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.axios.get(this.databaseUrl + "_design/" + designDocName + "/_view/" + viewName, {
                            params: __assign({}, this.encodeOptions(options), { reduce: false, include_docs: true })
                        })];
                    case 1:
                        result = _a.sent();
                        return [4 /*yield*/, this.checkErrorAndGetBody(result)];
                    case 2:
                        body = _a.sent();
                        return [2 /*return*/, body];
                }
            });
        });
    };
    /**
     * Creates the database associated with this client.
     */
    Client.prototype.createDb = function (url) {
        if (url === void 0) { url = this.databaseUrl; }
        return __awaiter(this, void 0, void 0, function () {
            var result, body;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.axios.put(url)];
                    case 1:
                        result = _a.sent();
                        if (result.status === 412) {
                            return [2 /*return*/, {
                                    ok: true,
                                    alreadyExisted: true
                                }];
                        }
                        return [4 /*yield*/, this.checkErrorAndGetBody(result)];
                    case 2:
                        body = _a.sent();
                        return [2 /*return*/, __assign({}, body, { alreadyExisted: false })];
                }
            });
        });
    };
    /**
     * Deletes the database associated with this client.
     */
    Client.prototype.deleteDb = function (url) {
        if (url === void 0) { url = this.databaseUrl; }
        return __awaiter(this, void 0, void 0, function () {
            var result;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.axios.delete(url)];
                    case 1:
                        result = _a.sent();
                        return [4 /*yield*/, this.checkErrorAndGetBody(result)];
                    case 2: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    /**
     * Returns database info for the given database.
     */
    Client.prototype.getDbInfo = function (url) {
        if (url === void 0) { url = this.databaseUrl; }
        return __awaiter(this, void 0, void 0, function () {
            var result;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.axios.get(url)];
                    case 1:
                        result = _a.sent();
                        return [4 /*yield*/, this.checkErrorAndGetBody(result)];
                    case 2: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    Client.prototype.encodeOptions = function (options) {
        var keys = Object.getOwnPropertyNames(options || {});
        return keys.reduce(function (requestOptions, key) {
            switch (key) {
                case "keys":
                case "key":
                case "start_key":
                case "end_key":
                    requestOptions[key] = JSON.stringify(options[key]);
                    break;
                default:
                    requestOptions[key] = options[key];
                    break;
            }
            return requestOptions;
        }, {});
    };
    return Client;
}());
exports.Client = Client;
exports.default = Client;
var DavenportError = (function (_super) {
    __extends(DavenportError, _super);
    function DavenportError(message, fullResponse) {
        var _this = _super.call(this, message) || this;
        _this.fullResponse = fullResponse;
        _this.isDavenport = true;
        _this.status = fullResponse.status;
        _this.statusText = fullResponse.statusText;
        _this.body = fullResponse.data;
        _this.url = fullResponse.headers.host || fullResponse.headers.HOST;
        return _this;
    }
    return DavenportError;
}(Error));
exports.DavenportError = DavenportError;
;
//# sourceMappingURL=index.js.map